@@ -8,14 +8,16 @@
 #include "src/handles/global-handles.h"
 #include "src/heap/array-buffer-sweeper.h"
 #include "src/heap/concurrent-allocator.h"
+#include "src/heap/concurrent-marking.h"
+#include "src/heap/ephemeron-remembered-set.h"
 #include "src/heap/gc-tracer-inl.h"
 #include "src/heap/gc-tracer.h"
 #include "src/heap/heap-inl.h"
 #include "src/heap/heap.h"
-#include "src/heap/invalidated-slots-inl.h"
 #include "src/heap/mark-compact-inl.h"
 #include "src/heap/mark-compact.h"
 #include "src/heap/memory-chunk-inl.h"
+#include "src/heap/memory-chunk-layout.h"
 #include "src/heap/memory-chunk.h"
 #include "src/heap/objects-visiting-inl.h"
 #include "src/heap/pretenuring-handler.h"
@@ -40,48 +42,30 @@ class IterateAndScavengePromotedObjectsVisitor final : public ObjectVisitor {
                                            bool record_slots)
       : scavenger_(scavenger), record_slots_(record_slots) {}
 
-  V8_INLINE void VisitMapPointer(HeapObject host) final {
+  V8_INLINE void VisitMapPointer(Tagged<HeapObject> host) final {
     if (!record_slots_) return;
-    MapWord map_word = host.map_word(kRelaxedLoad);
+    MapWord map_word = host->map_word(kRelaxedLoad);
     if (map_word.IsForwardingAddress()) {
       // Surviving new large objects have forwarding pointers in the map word.
       DCHECK(MemoryChunk::FromHeapObject(host)->InNewLargeObjectSpace());
       return;
     }
-    HandleSlot(host, HeapObjectSlot(host.map_slot()), map_word.ToMap());
+    HandleSlot(host, HeapObjectSlot(host->map_slot()), map_word.ToMap());
   }
 
-  V8_INLINE void VisitPointers(HeapObject host, ObjectSlot start,
+  V8_INLINE void VisitPointers(Tagged<HeapObject> host, ObjectSlot start,
                                ObjectSlot end) final {
     VisitPointersImpl(host, start, end);
   }
 
-  V8_INLINE void VisitPointers(HeapObject host, MaybeObjectSlot start,
+  V8_INLINE void VisitPointers(Tagged<HeapObject> host, MaybeObjectSlot start,
                                MaybeObjectSlot end) final {
     VisitPointersImpl(host, start, end);
   }
 
-  V8_INLINE void VisitCodePointer(HeapObject host, CodeObjectSlot slot) final {
-    CHECK(V8_EXTERNAL_CODE_SPACE_BOOL);
-    // Code slots never appear in new space because CodeDataContainers, the
-    // only object that can contain code pointers, are always allocated in
-    // the old space.
-    UNREACHABLE();
-  }
-
-  V8_INLINE void VisitCodeTarget(Code host, RelocInfo* rinfo) final {
-    Code target = Code::GetCodeFromTargetAddress(rinfo->target_address());
-    HandleSlot(host, FullHeapObjectSlot(&target), target);
-  }
-  V8_INLINE void VisitEmbeddedPointer(Code host, RelocInfo* rinfo) final {
-    PtrComprCageBase cage_base = host.main_cage_base();
-    HeapObject heap_object = rinfo->target_object(cage_base);
-    HandleSlot(host, FullHeapObjectSlot(&heap_object), heap_object);
-  }
-
-  inline void VisitEphemeron(HeapObject obj, int entry, ObjectSlot key,
+  inline void VisitEphemeron(Tagged<HeapObject> obj, int entry, ObjectSlot key,
                              ObjectSlot value) override {
-    DCHECK(Heap::IsLargeObject(obj) || obj.IsEphemeronHashTable());
+    DCHECK(Heap::IsLargeObject(obj) || IsEphemeronHashTable(obj));
     VisitPointer(obj, value);
 
     if (ObjectInYoungGeneration(*key)) {
@@ -93,15 +77,29 @@ class IterateAndScavengePromotedObjectsVisitor final : public ObjectVisitor {
     }
   }
 
+  // Special cases: Unreachable visitors for objects that are never found in the
+  // young generation and thus cannot be found when iterating promoted objects.
+  void VisitInstructionStreamPointer(Tagged<Code>,
+                                     InstructionStreamSlot) final {
+    UNREACHABLE();
+  }
+  void VisitCodeTarget(Tagged<InstructionStream>, RelocInfo*) final {
+    UNREACHABLE();
+  }
+  void VisitEmbeddedPointer(Tagged<InstructionStream>, RelocInfo*) final {
+    UNREACHABLE();
+  }
+
  private:
   template <typename TSlot>
-  V8_INLINE void VisitPointersImpl(HeapObject host, TSlot start, TSlot end) {
+  V8_INLINE void VisitPointersImpl(Tagged<HeapObject> host, TSlot start,
+                                   TSlot end) {
     using THeapObjectSlot = typename TSlot::THeapObjectSlot;
     // Treat weak references as strong.
     // TODO(marja): Proper weakness handling in the young generation.
     for (TSlot slot = start; slot < end; ++slot) {
       typename TSlot::TObject object = *slot;
-      HeapObject heap_object;
+      Tagged<HeapObject> heap_object;
       if (object.GetHeapObject(&heap_object)) {
         HandleSlot(host, THeapObjectSlot(slot), heap_object);
       }
@@ -109,8 +107,8 @@ class IterateAndScavengePromotedObjectsVisitor final : public ObjectVisitor {
   }
 
   template <typename THeapObjectSlot>
-  V8_INLINE void HandleSlot(HeapObject host, THeapObjectSlot slot,
-                            HeapObject target) {
+  V8_INLINE void HandleSlot(Tagged<HeapObject> host, THeapObjectSlot slot,
+                            Tagged<HeapObject> target) {
     static_assert(
         std::is_same<THeapObjectSlot, FullHeapObjectSlot>::value ||
             std::is_same<THeapObjectSlot, HeapObjectSlot>::value,
@@ -119,12 +117,12 @@ class IterateAndScavengePromotedObjectsVisitor final : public ObjectVisitor {
 
     if (Heap::InFromPage(target)) {
       SlotCallbackResult result = scavenger_->ScavengeObject(slot, target);
-      bool success = (*slot)->GetHeapObject(&target);
+      bool success = (*slot).GetHeapObject(&target);
       USE(success);
       DCHECK(success);
 
       if (result == KEEP_SLOT) {
-        SLOW_DCHECK(target.IsHeapObject());
+        SLOW_DCHECK(IsHeapObject(target));
         MemoryChunk* chunk = MemoryChunk::FromHeapObject(host);
 
         // Sweeper is stopped during scavenge, so we can directly
@@ -132,20 +130,17 @@ class IterateAndScavengePromotedObjectsVisitor final : public ObjectVisitor {
         RememberedSet<OLD_TO_NEW>::Insert<AccessMode::ATOMIC>(chunk,
                                                               slot.address());
       }
-      SLOW_DCHECK(!MarkCompactCollector::IsOnEvacuationCandidate(target));
+      DCHECK(!MarkCompactCollector::IsOnEvacuationCandidate(target));
     } else if (record_slots_ &&
                MarkCompactCollector::IsOnEvacuationCandidate(target)) {
       // We should never try to record off-heap slots.
       DCHECK((std::is_same<THeapObjectSlot, HeapObjectSlot>::value));
-      // Code slots never appear in new space because CodeDataContainers, the
-      // only object that can contain code pointers, are always allocated in
-      // the old space.
+      // InstructionStream slots never appear in new space because
+      // Code objects, the only object that can contain code pointers, are
+      // always allocated in the old space.
       DCHECK_IMPLIES(V8_EXTERNAL_CODE_SPACE_BOOL,
                      !MemoryChunk::FromHeapObject(target)->IsFlagSet(
                          MemoryChunk::IS_EXECUTABLE));
-      // Shared heap pages do not have evacuation candidates outside an atomic
-      // shared GC pause.
-      DCHECK(!target.InSharedWritableHeap());
 
       // We cannot call MarkCompactCollector::RecordSlot because that checks
       // that the host page is not in young generation, which does not hold
@@ -154,8 +149,7 @@ class IterateAndScavengePromotedObjectsVisitor final : public ObjectVisitor {
           MemoryChunk::FromHeapObject(host), slot.address());
     }
 
-    if (target.InSharedWritableHeap()) {
-      DCHECK(!scavenger_->heap()->IsShared());
+    if (target.InWritableSharedSpace()) {
       MemoryChunk* chunk = MemoryChunk::FromHeapObject(host);
       RememberedSet<OLD_TO_SHARED>::Insert<AccessMode::ATOMIC>(chunk,
                                                                slot.address());
@@ -168,15 +162,17 @@ class IterateAndScavengePromotedObjectsVisitor final : public ObjectVisitor {
 
 namespace {
 
-V8_INLINE bool IsUnscavengedHeapObject(Heap* heap, Object object) {
-  return Heap::InFromPage(object) &&
-         !HeapObject::cast(object).map_word(kRelaxedLoad).IsForwardingAddress();
+V8_INLINE bool IsUnscavengedHeapObject(Heap* heap, Tagged<Object> object) {
+  return Heap::InFromPage(object) && !HeapObject::cast(object)
+                                          ->map_word(kRelaxedLoad)
+                                          .IsForwardingAddress();
 }
 
 // Same as IsUnscavengedHeapObject() above but specialized for HeapObjects.
-V8_INLINE bool IsUnscavengedHeapObject(Heap* heap, HeapObject heap_object) {
+V8_INLINE bool IsUnscavengedHeapObject(Heap* heap,
+                                       Tagged<HeapObject> heap_object) {
   return Heap::InFromPage(heap_object) &&
-         !heap_object.map_word(kRelaxedLoad).IsForwardingAddress();
+         !heap_object->map_word(kRelaxedLoad).IsForwardingAddress();
 }
 
 bool IsUnscavengedHeapObjectSlot(Heap* heap, FullObjectSlot p) {
@@ -197,22 +193,24 @@ ScavengerCollector::JobTask::JobTask(
       remaining_memory_chunks_(memory_chunks_.size()),
       generator_(memory_chunks_.size()),
       copied_list_(copied_list),
-      promotion_list_(promotion_list) {}
+      promotion_list_(promotion_list),
+      trace_id_(
+          reinterpret_cast<uint64_t>(this) ^
+          outer_->heap_->tracer()->CurrentEpoch(GCTracer::Scope::SCAVENGER)) {}
 
 void ScavengerCollector::JobTask::Run(JobDelegate* delegate) {
-  // The task accesses code pages and thus the permissions must be set to
-  // default state.
-  RwxMemoryWriteScope::SetDefaultPermissionsForNewThread();
   DCHECK_LT(delegate->GetTaskId(), scavengers_->size());
   Scavenger* scavenger = (*scavengers_)[delegate->GetTaskId()].get();
   if (delegate->IsJoiningThread()) {
-    // This is already traced in GCTracer::Scope::SCAVENGER_SCAVENGE_PARALLEL
-    // in ScavengerCollector::CollectGarbage.
+    TRACE_GC_WITH_FLOW(outer_->heap_->tracer(),
+                       GCTracer::Scope::SCAVENGER_SCAVENGE_PARALLEL, trace_id_,
+                       TRACE_EVENT_FLAG_FLOW_IN);
     ProcessItems(delegate, scavenger);
   } else {
-    TRACE_GC_EPOCH(outer_->heap_->tracer(),
-                   GCTracer::Scope::SCAVENGER_BACKGROUND_SCAVENGE_PARALLEL,
-                   ThreadKind::kBackground);
+    TRACE_GC_EPOCH_WITH_FLOW(
+        outer_->heap_->tracer(),
+        GCTracer::Scope::SCAVENGER_BACKGROUND_SCAVENGE_PARALLEL,
+        ThreadKind::kBackground, trace_id_, TRACE_EVENT_FLAG_FLOW_IN);
     ProcessItems(delegate, scavenger);
   }
 }
@@ -221,11 +219,13 @@ size_t ScavengerCollector::JobTask::GetMaxConcurrency(
     size_t worker_count) const {
   // We need to account for local segments held by worker_count in addition to
   // GlobalPoolSize() of copied_list_ and promotion_list_.
-  return std::min<size_t>(
-      scavengers_->size(),
-      std::max<size_t>(
-          remaining_memory_chunks_.load(std::memory_order_relaxed),
-          worker_count + copied_list_->Size() + promotion_list_->Size()));
+  size_t wanted_num_workers = std::max<size_t>(
+      remaining_memory_chunks_.load(std::memory_order_relaxed),
+      worker_count + copied_list_->Size() + promotion_list_->Size());
+  if (!outer_->heap_->ShouldUseBackgroundThreads()) {
+    return std::min<size_t>(wanted_num_workers, 1);
+  }
+  return std::min<size_t>(scavengers_->size(), wanted_num_workers);
 }
 
 void ScavengerCollector::JobTask::ProcessItems(JobDelegate* delegate,
@@ -283,20 +283,20 @@ class GlobalHandlesWeakRootsUpdatingVisitor final : public RootVisitor {
 
  private:
   void UpdatePointer(FullObjectSlot p) {
-    Object object = *p;
+    Tagged<Object> object = *p;
     DCHECK(!HasWeakHeapObjectTag(object));
     // The object may be in the old generation as global handles over
     // approximates the list of young nodes. This checks also bails out for
     // Smis.
     if (!Heap::InYoungGeneration(object)) return;
 
-    HeapObject heap_object = HeapObject::cast(object);
+    Tagged<HeapObject> heap_object = HeapObject::cast(object);
     // TODO(chromium:1336158): Turn the following CHECKs into DCHECKs after
     // flushing out potential issues.
     CHECK(Heap::InFromPage(heap_object));
-    MapWord first_word = heap_object.map_word(kRelaxedLoad);
+    MapWord first_word = heap_object->map_word(kRelaxedLoad);
     CHECK(first_word.IsForwardingAddress());
-    HeapObject dest = first_word.ToForwardingAddress();
+    Tagged<HeapObject> dest = first_word.ToForwardingAddress(heap_object);
     HeapObjectReference::Update(FullHeapObjectSlot(p), dest);
     CHECK_IMPLIES(Heap::InYoungGeneration(dest),
                   Heap::InToPage(dest) || Heap::IsLargeObject(dest));
@@ -328,22 +328,9 @@ void ScavengerCollector::CollectGarbage() {
   const int num_scavenge_tasks = NumberOfScavengeTasks();
   Scavenger::CopiedList copied_list;
   Scavenger::PromotionList promotion_list;
-  EphemeronTableList ephemeron_table_list;
+  EphemeronRememberedSet::TableList ephemeron_table_list;
 
   {
-    Sweeper* sweeper = heap_->sweeper();
-
-    // Pause the concurrent sweeper.
-    Sweeper::PauseScope pause_scope(sweeper);
-    // Filter out pages from the sweeper that need to be processed for old to
-    // new slots by the Scavenger. After processing, the Scavenger adds back
-    // pages that are still unsweeped. This way the Scavenger has exclusive
-    // access to the slots of a page and can completely avoid any locks on
-    // the page itself.
-    Sweeper::FilterSweepingPagesScope filter_scope(sweeper, pause_scope);
-    filter_scope.FilterOldSpaceSweepingPages(
-        [](Page* page) { return !page->ContainsSlots<OLD_TO_NEW>(); });
-
     const bool is_logging = isolate_->log_object_relocation();
     for (int i = 0; i < num_scavenge_tasks; ++i) {
       scavengers.emplace_back(
@@ -352,9 +339,13 @@ void ScavengerCollector::CollectGarbage() {
     }
 
     std::vector<std::pair<ParallelWorkItem, MemoryChunk*>> memory_chunks;
-    RememberedSet<OLD_TO_NEW>::IterateMemoryChunks(
+    OldGenerationMemoryChunkIterator::ForAll(
         heap_, [&memory_chunks](MemoryChunk* chunk) {
-          memory_chunks.emplace_back(ParallelWorkItem{}, chunk);
+          if (chunk->slot_set<OLD_TO_NEW>() ||
+              chunk->typed_slot_set<OLD_TO_NEW>() ||
+              chunk->slot_set<OLD_TO_NEW_BACKGROUND>()) {
+            memory_chunks.emplace_back(ParallelWorkItem{}, chunk);
+          }
         });
 
     RootScavengeVisitor root_scavenge_visitor(scavengers[kMainThreadId].get());
@@ -364,7 +355,7 @@ void ScavengerCollector::CollectGarbage() {
       TRACE_GC(
           heap_->tracer(),
           GCTracer::Scope::SCAVENGER_SCAVENGE_WEAK_GLOBAL_HANDLES_IDENTIFY);
-      isolate_->global_handles()->ComputeWeaknessForYoungObjects(
+      isolate_->traced_handles()->ComputeWeaknessForYoungObjects(
           &JSObject::IsUnmodifiedApiObject);
     }
     {
@@ -373,25 +364,31 @@ void ScavengerCollector::CollectGarbage() {
       // Scavenger treats all weak roots except for global handles as strong.
       // That is why we don't set skip_weak = true here and instead visit
       // global handles separately.
-      base::EnumSet<SkipRoot> options({SkipRoot::kExternalStringTable,
-                                       SkipRoot::kGlobalHandles,
-                                       SkipRoot::kOldGeneration});
+      base::EnumSet<SkipRoot> options(
+          {SkipRoot::kExternalStringTable, SkipRoot::kGlobalHandles,
+           SkipRoot::kTracedHandles, SkipRoot::kOldGeneration,
+           SkipRoot::kConservativeStack, SkipRoot::kReadOnlyBuiltins});
       if (V8_UNLIKELY(v8_flags.scavenge_separate_stack_scanning)) {
         options.Add(SkipRoot::kStack);
       }
       heap_->IterateRoots(&root_scavenge_visitor, options);
       isolate_->global_handles()->IterateYoungStrongAndDependentRoots(
           &root_scavenge_visitor);
+      isolate_->traced_handles()->IterateYoungRoots(&root_scavenge_visitor);
       scavengers[kMainThreadId]->Publish();
     }
     {
       // Parallel phase scavenging all copied and promoted objects.
-      TRACE_GC(heap_->tracer(), GCTracer::Scope::SCAVENGER_SCAVENGE_PARALLEL);
+      TRACE_GC_ARG1(
+          heap_->tracer(), GCTracer::Scope::SCAVENGER_SCAVENGE_PARALLEL_PHASE,
+          "UseBackgroundThreads", heap_->ShouldUseBackgroundThreads());
+      auto job =
+          std::make_unique<JobTask>(this, &scavengers, std::move(memory_chunks),
+                                    &copied_list, &promotion_list);
+      TRACE_GC_NOTE_WITH_FLOW("Parallel scavenge started", job->trace_id(),
+                              TRACE_EVENT_FLAG_FLOW_OUT);
       V8::GetCurrentPlatform()
-          ->CreateJob(v8::TaskPriority::kUserBlocking,
-                      std::make_unique<JobTask>(this, &scavengers,
-                                                std::move(memory_chunks),
-                                                &copied_list, &promotion_list))
+          ->CreateJob(v8::TaskPriority::kUserBlocking, std::move(job))
           ->Join();
       DCHECK(copied_list.IsEmpty());
       DCHECK(promotion_list.IsEmpty());
@@ -411,6 +408,8 @@ void ScavengerCollector::CollectGarbage() {
       GlobalHandlesWeakRootsUpdatingVisitor visitor;
       isolate_->global_handles()->ProcessWeakYoungObjects(
           &visitor, &IsUnscavengedHeapObjectSlot);
+      isolate_->traced_handles()->ProcessYoungObjects(
+          &visitor, &IsUnscavengedHeapObjectSlot);
     }
 
     {
@@ -434,14 +433,14 @@ void ScavengerCollector::CollectGarbage() {
     heap_->UpdateYoungReferencesInExternalStringTable(
         &Heap::UpdateYoungReferenceInExternalStringTableEntry);
 
-    heap_->incremental_marking()->UpdateMarkingWorklistAfterYoungGenGC();
+    heap_->incremental_marking()->UpdateMarkingWorklistAfterScavenge();
 
     if (V8_UNLIKELY(v8_flags.track_retaining_path)) {
       heap_->UpdateRetainersAfterScavenge();
     }
 
     if (V8_UNLIKELY(v8_flags.always_use_string_forwarding_table)) {
-      isolate_->string_forwarding_table()->UpdateAfterEvacuation();
+      isolate_->string_forwarding_table()->UpdateAfterYoungEvacuation();
     }
   }
 
@@ -465,7 +464,8 @@ void ScavengerCollector::CollectGarbage() {
   // Since we promote all surviving large objects immediately, all remaining
   // large objects must be dead.
   // TODO(hpayer): Don't free all as soon as we have an intermediate generation.
-  heap_->new_lo_space()->FreeDeadObjects([](HeapObject) { return true; });
+  heap_->new_lo_space()->FreeDeadObjects(
+      [](Tagged<HeapObject>) { return true; });
 
   {
     TRACE_GC(heap_->tracer(), GCTracer::Scope::SCAVENGER_FREE_REMEMBERED_SET);
@@ -476,15 +476,20 @@ void ScavengerCollector::CollectGarbage() {
       // swept.
       if (chunk->SweepingDone()) {
         RememberedSet<OLD_TO_NEW>::CheckPossiblyEmptyBuckets(chunk);
+        RememberedSet<OLD_TO_NEW_BACKGROUND>::CheckPossiblyEmptyBuckets(chunk);
       } else {
         chunk->possibly_empty_buckets()->Release();
       }
     }
 
 #ifdef DEBUG
-    RememberedSet<OLD_TO_NEW>::IterateMemoryChunks(
+    OldGenerationMemoryChunkIterator::ForAll(
         heap_, [](MemoryChunk* chunk) {
-          DCHECK(chunk->possibly_empty_buckets()->IsEmpty());
+          if (chunk->slot_set<OLD_TO_NEW>() ||
+              chunk->typed_slot_set<OLD_TO_NEW>() ||
+              chunk->slot_set<OLD_TO_NEW_BACKGROUND>()) {
+            DCHECK(chunk->possibly_empty_buckets()->IsEmpty());
+          }
         });
 #endif
   }
@@ -495,13 +500,13 @@ void ScavengerCollector::CollectGarbage() {
   }
 
   isolate_->global_handles()->UpdateListOfYoungNodes();
+  isolate_->traced_handles()->UpdateListOfYoungNodes();
 
   // Update how much has survived scavenge.
   heap_->IncrementYoungSurvivorsCounter(heap_->SurvivedYoungObjectSize());
 }
 
 void ScavengerCollector::IterateStackAndScavenge(
-
     RootScavengeVisitor* root_scavenge_visitor,
     std::vector<std::unique_ptr<Scavenger>>* scavengers, int main_thread_id) {
   // Scan the stack, scavenge the newly discovered objects, and report
@@ -535,26 +540,30 @@ void ScavengerCollector::IterateStackAndScavenge(
 }
 
 void ScavengerCollector::SweepArrayBufferExtensions() {
+  DCHECK_EQ(0, heap_->new_lo_space()->Size());
   heap_->array_buffer_sweeper()->RequestSweep(
-      ArrayBufferSweeper::SweepingType::kYoung);
+      ArrayBufferSweeper::SweepingType::kYoung,
+      (heap_->new_space()->Size() == 0)
+          ? ArrayBufferSweeper::TreatAllYoungAsPromoted::kYes
+          : ArrayBufferSweeper::TreatAllYoungAsPromoted::kNo);
 }
 
 void ScavengerCollector::HandleSurvivingNewLargeObjects() {
   const bool is_compacting = heap_->incremental_marking()->IsCompacting();
-  AtomicMarkingState* marking_state = heap_->atomic_marking_state();
+  MarkingState* marking_state = heap_->marking_state();
 
   for (SurvivingNewLargeObjectMapEntry update_info :
        surviving_new_large_objects_) {
-    HeapObject object = update_info.first;
-    Map map = update_info.second;
+    Tagged<HeapObject> object = update_info.first;
+    Tagged<Map> map = update_info.second;
     // Order is important here. We have to re-install the map to have access
     // to meta-data like size during page promotion.
-    object.set_map_word(MapWord::FromMap(map), kRelaxedStore);
+    object->set_map_word(map, kRelaxedStore);
 
-    if (is_compacting && marking_state->IsBlack(object) &&
+    if (is_compacting && marking_state->IsMarked(object) &&
         MarkCompactCollector::IsOnEvacuationCandidate(map)) {
       RememberedSet<OLD_TO_OLD>::Insert<AccessMode::ATOMIC>(
-          MemoryChunk::FromHeapObject(object), object.map_slot().address());
+          MemoryChunk::FromHeapObject(object), object->map_slot().address());
     }
     LargePage* page = LargePage::FromHeapObject(object);
     heap_->lo_space()->PromoteNewLargeObject(page);
@@ -598,17 +607,20 @@ Scavenger::PromotionList::Local::Local(Scavenger::PromotionList* promotion_list)
 
 namespace {
 ConcurrentAllocator* CreateSharedOldAllocator2(Heap* heap) {
-  if (v8_flags.shared_string_table && heap->isolate()->has_shared_heap()) {
-    return new ConcurrentAllocator(nullptr, heap->shared_allocation_space());
+  if (v8_flags.shared_string_table && heap->isolate()->has_shared_space()) {
+    return new ConcurrentAllocator(nullptr, heap->shared_allocation_space(),
+                                   ConcurrentAllocator::Context::kGC);
   }
   return nullptr;
 }
+
 }  // namespace
 
 Scavenger::Scavenger(ScavengerCollector* collector, Heap* heap, bool is_logging,
                      EmptyChunksList* empty_chunks, CopiedList* copied_list,
                      PromotionList* promotion_list,
-                     EphemeronTableList* ephemeron_table_list, int task_id)
+                     EphemeronRememberedSet::TableList* ephemeron_table_list,
+                     int task_id)
     : collector_(collector),
       heap_(heap),
       empty_chunks_local_(*empty_chunks),
@@ -616,8 +628,7 @@ Scavenger::Scavenger(ScavengerCollector* collector, Heap* heap, bool is_logging,
       copied_list_local_(*copied_list),
       ephemeron_table_list_local_(*ephemeron_table_list),
       pretenuring_handler_(heap_->pretenuring_handler()),
-      local_pretenuring_feedback_(
-          PretenturingHandler::kInitialFeedbackCapacity),
+      local_pretenuring_feedback_(PretenuringHandler::kInitialFeedbackCapacity),
       copied_size_(0),
       promoted_size_(0),
       allocator_(heap, CompactionSpaceKind::kCompactionSpaceForScavenge),
@@ -625,13 +636,16 @@ Scavenger::Scavenger(ScavengerCollector* collector, Heap* heap, bool is_logging,
       is_logging_(is_logging),
       is_incremental_marking_(heap->incremental_marking()->IsMarking()),
       is_compacting_(heap->incremental_marking()->IsCompacting()),
-      is_compacting_including_map_space_(is_compacting_ &&
-                                         v8_flags.compact_maps),
-      shared_string_table_(shared_old_allocator_.get() != nullptr),
-      mark_shared_heap_(heap->isolate()->is_shared_space_isolate()) {}
+      shared_string_table_(shared_old_allocator_ != nullptr),
+      mark_shared_heap_(heap->isolate()->is_shared_space_isolate()),
+      shortcut_strings_(
+          heap->CanShortcutStringsDuringGC(GarbageCollector::SCAVENGER)) {
+  DCHECK_IMPLIES(is_incremental_marking_,
+                 heap->incremental_marking()->IsMajorMarking());
+}
 
-void Scavenger::IterateAndScavengePromotedObject(HeapObject target, Map map,
-                                                 int size) {
+void Scavenger::IterateAndScavengePromotedObject(Tagged<HeapObject> target,
+                                                 Tagged<Map> map, int size) {
   // We are not collecting slots on new space objects during mutation thus we
   // have to scan for pointers to evacuation candidates when we promote
   // objects. But we should not record any slots in non-black objects. Grey
@@ -639,53 +653,38 @@ void Scavenger::IterateAndScavengePromotedObject(HeapObject target, Map map,
   // the end of collection it would be a violation of the invariant to record
   // its slots.
   const bool record_slots =
-      is_compacting_ && heap()->atomic_marking_state()->IsBlack(target);
+      is_compacting_ && heap()->marking_state()->IsMarked(target);
 
   IterateAndScavengePromotedObjectsVisitor visitor(this, record_slots);
 
-  if (is_compacting_including_map_space_) {
-    // When we compact map space, we also want to visit the map word.
-    target.IterateFast(map, size, &visitor);
-  } else {
-    target.IterateBodyFast(map, size, &visitor);
-  }
+  // Iterate all outgoing pointers including map word.
+  target->IterateFast(map, size, &visitor);
 
-  if (map.IsJSArrayBufferMap()) {
+  if (IsJSArrayBufferMap(map)) {
     DCHECK(!BasicMemoryChunk::FromHeapObject(target)->IsLargePage());
-    JSArrayBuffer::cast(target).YoungMarkExtensionPromoted();
+    JSArrayBuffer::cast(target)->YoungMarkExtensionPromoted();
   }
 }
 
-void Scavenger::RememberPromotedEphemeron(EphemeronHashTable table, int entry) {
+void Scavenger::RememberPromotedEphemeron(Tagged<EphemeronHashTable> table,
+                                          int index) {
   auto indices =
       ephemeron_remembered_set_.insert({table, std::unordered_set<int>()});
-  indices.first->second.insert(entry);
-}
-
-void Scavenger::AddPageToSweeperIfNecessary(MemoryChunk* page) {
-  AllocationSpace space = page->owner_identity();
-  if ((space == OLD_SPACE) && !page->SweepingDone()) {
-    heap()->sweeper()->AddPage(space, reinterpret_cast<Page*>(page),
-                               Sweeper::READD_TEMPORARY_REMOVED_PAGE);
-  }
+  indices.first->second.insert(index);
 }
 
 void Scavenger::ScavengePage(MemoryChunk* page) {
   CodePageMemoryModificationScope memory_modification_scope(page);
-  const bool record_old_to_shared_slots = heap_->isolate()->has_shared_heap();
+  const bool record_old_to_shared_slots = heap_->isolate()->has_shared_space();
 
   if (page->slot_set<OLD_TO_NEW, AccessMode::ATOMIC>() != nullptr) {
-    InvalidatedSlotsFilter filter = InvalidatedSlotsFilter::OldToNew(
-        page, InvalidatedSlotsFilter::LivenessCheck::kNo);
     RememberedSet<OLD_TO_NEW>::IterateAndTrackEmptyBuckets(
         page,
-        [this, page, record_old_to_shared_slots,
-         &filter](MaybeObjectSlot slot) {
-          if (!filter.IsValid(slot.address())) return REMOVE_SLOT;
+        [this, page, record_old_to_shared_slots](MaybeObjectSlot slot) {
           SlotCallbackResult result = CheckAndScavengeObject(heap_, slot);
           // A new space string might have been promoted into the shared heap
           // during GC.
-          if (record_old_to_shared_slots) {
+          if (result == REMOVE_SLOT && record_old_to_shared_slots) {
             CheckOldToNewSlotForSharedUntyped(page, slot);
           }
           return result;
@@ -693,14 +692,9 @@ void Scavenger::ScavengePage(MemoryChunk* page) {
         &empty_chunks_local_);
   }
 
-  if (page->invalidated_slots<OLD_TO_NEW>() != nullptr) {
-    // The invalidated slots are not needed after old-to-new slots were
-    // processed.
-    page->ReleaseInvalidatedSlots<OLD_TO_NEW>();
-  }
-
   RememberedSet<OLD_TO_NEW>::IterateTyped(
-      page, [=](SlotType slot_type, Address slot_address) {
+      page, [this, page, record_old_to_shared_slots](SlotType slot_type,
+                                                     Address slot_address) {
         return UpdateTypedSlotHelper::UpdateTypedSlot(
             heap_, slot_type, slot_address,
             [this, page, slot_type, slot_address,
@@ -708,14 +702,28 @@ void Scavenger::ScavengePage(MemoryChunk* page) {
               SlotCallbackResult result = CheckAndScavengeObject(heap(), slot);
               // A new space string might have been promoted into the shared
               // heap during GC.
-              if (record_old_to_shared_slots) {
-                CheckOldToNewSlotForSharedTyped(page, slot_type, slot_address);
+              if (result == REMOVE_SLOT && record_old_to_shared_slots) {
+                CheckOldToNewSlotForSharedTyped(page, slot_type, slot_address,
+                                                *slot);
               }
               return result;
             });
       });
 
-  AddPageToSweeperIfNecessary(page);
+  if (page->slot_set<OLD_TO_NEW_BACKGROUND, AccessMode::ATOMIC>() != nullptr) {
+    RememberedSet<OLD_TO_NEW_BACKGROUND>::IterateAndTrackEmptyBuckets(
+        page,
+        [this, page, record_old_to_shared_slots](MaybeObjectSlot slot) {
+          SlotCallbackResult result = CheckAndScavengeObject(heap_, slot);
+          // A new space string might have been promoted into the shared heap
+          // during GC.
+          if (result == REMOVE_SLOT && record_old_to_shared_slots) {
+            CheckOldToNewSlotForSharedUntyped(page, slot);
+          }
+          return result;
+        },
+        &empty_chunks_local_);
+  }
 }
 
 void Scavenger::Process(JobDelegate* delegate) {
@@ -739,7 +747,7 @@ void Scavenger::Process(JobDelegate* delegate) {
 
     struct PromotionListEntry entry;
     while (promotion_list_local_.Pop(&entry)) {
-      HeapObject target = entry.heap_object;
+      Tagged<HeapObject> target = entry.heap_object;
       IterateAndScavengePromotedObject(target, entry.map, entry.size);
       done = false;
       if (delegate && ((++objects % kInterruptThreshold) == 0)) {
@@ -752,7 +760,7 @@ void Scavenger::Process(JobDelegate* delegate) {
 }
 
 void ScavengerCollector::ProcessWeakReferences(
-    EphemeronTableList* ephemeron_table_list) {
+    EphemeronRememberedSet::TableList* ephemeron_table_list) {
   ClearYoungEphemerons(ephemeron_table_list);
   ClearOldEphemerons();
 }
@@ -760,17 +768,17 @@ void ScavengerCollector::ProcessWeakReferences(
 // Clear ephemeron entries from EphemeronHashTables in new-space whenever the
 // entry has a dead new-space key.
 void ScavengerCollector::ClearYoungEphemerons(
-    EphemeronTableList* ephemeron_table_list) {
-  ephemeron_table_list->Iterate([this](EphemeronHashTable table) {
-    for (InternalIndex i : table.IterateEntries()) {
+    EphemeronRememberedSet::TableList* ephemeron_table_list) {
+  ephemeron_table_list->Iterate([this](Tagged<EphemeronHashTable> table) {
+    for (InternalIndex i : table->IterateEntries()) {
       // Keys in EphemeronHashTables must be heap objects.
       HeapObjectSlot key_slot(
-          table.RawFieldOfElementAt(EphemeronHashTable::EntryToIndex(i)));
-      HeapObject key = key_slot.ToHeapObject();
+          table->RawFieldOfElementAt(EphemeronHashTable::EntryToIndex(i)));
+      Tagged<HeapObject> key = key_slot.ToHeapObject();
       if (IsUnscavengedHeapObject(heap_, key)) {
-        table.RemoveEntry(i);
+        table->RemoveEntry(i);
       } else {
-        HeapObject forwarded = ForwardingAddress(key);
+        Tagged<HeapObject> forwarded = ForwardingAddress(key);
         key_slot.StoreHeapObject(forwarded);
       }
     }
@@ -781,20 +789,20 @@ void ScavengerCollector::ClearYoungEphemerons(
 // Clear ephemeron entries from EphemeronHashTables in old-space whenever the
 // entry has a dead new-space key.
 void ScavengerCollector::ClearOldEphemerons() {
-  for (auto it = heap_->ephemeron_remembered_set_.begin();
-       it != heap_->ephemeron_remembered_set_.end();) {
-    EphemeronHashTable table = it->first;
+  auto* table_map = heap_->ephemeron_remembered_set_->tables();
+  for (auto it = table_map->begin(); it != table_map->end();) {
+    Tagged<EphemeronHashTable> table = it->first;
     auto& indices = it->second;
     for (auto iti = indices.begin(); iti != indices.end();) {
       // Keys in EphemeronHashTables must be heap objects.
-      HeapObjectSlot key_slot(table.RawFieldOfElementAt(
+      HeapObjectSlot key_slot(table->RawFieldOfElementAt(
           EphemeronHashTable::EntryToIndex(InternalIndex(*iti))));
-      HeapObject key = key_slot.ToHeapObject();
+      Tagged<HeapObject> key = key_slot.ToHeapObject();
       if (IsUnscavengedHeapObject(heap_, key)) {
-        table.RemoveEntry(InternalIndex(*iti));
+        table->RemoveEntry(InternalIndex(*iti));
         iti = indices.erase(iti);
       } else {
-        HeapObject forwarded = ForwardingAddress(key);
+        Tagged<HeapObject> forwarded = ForwardingAddress(key);
         key_slot.StoreHeapObject(forwarded);
         if (!Heap::InYoungGeneration(forwarded)) {
           iti = indices.erase(iti);
@@ -804,8 +812,8 @@ void ScavengerCollector::ClearOldEphemerons() {
       }
     }
 
-    if (indices.size() == 0) {
-      it = heap_->ephemeron_remembered_set_.erase(it);
+    if (indices.empty()) {
+      it = table_map->erase(it);
     } else {
       ++it;
     }
@@ -824,11 +832,10 @@ void Scavenger::Finalize() {
   ephemeron_table_list_local_.Publish();
   for (auto it = ephemeron_remembered_set_.begin();
        it != ephemeron_remembered_set_.end(); ++it) {
-    auto insert_result = heap()->ephemeron_remembered_set_.insert(
-        {it->first, std::unordered_set<int>()});
-    for (int entry : it->second) {
-      insert_result.first->second.insert(entry);
-    }
+    DCHECK_IMPLIES(!MemoryChunk::FromHeapObject(it->first)->IsLargePage(),
+                   !Heap::InYoungGeneration(it->first));
+    heap()->ephemeron_remembered_set()->RecordEphemeronKeyWrites(
+        it->first, std::move(it->second));
   }
 }
 
@@ -837,7 +844,7 @@ void Scavenger::Publish() {
   promotion_list_local_.Publish();
 }
 
-void Scavenger::AddEphemeronHashTable(EphemeronHashTable table) {
+void Scavenger::AddEphemeronHashTable(Tagged<EphemeronHashTable> table) {
   ephemeron_table_list_local_.Push(table);
 }
 
@@ -845,10 +852,10 @@ template <typename TSlot>
 void Scavenger::CheckOldToNewSlotForSharedUntyped(MemoryChunk* chunk,
                                                   TSlot slot) {
   MaybeObject object = *slot;
-  HeapObject heap_object;
+  Tagged<HeapObject> heap_object;
 
   if (object.GetHeapObject(&heap_object) &&
-      heap_object.InSharedWritableHeap()) {
+      heap_object.InWritableSharedSpace()) {
     RememberedSet<OLD_TO_SHARED>::Insert<AccessMode::ATOMIC>(chunk,
                                                              slot.address());
   }
@@ -856,20 +863,12 @@ void Scavenger::CheckOldToNewSlotForSharedUntyped(MemoryChunk* chunk,
 
 void Scavenger::CheckOldToNewSlotForSharedTyped(MemoryChunk* chunk,
                                                 SlotType slot_type,
-                                                Address slot_address) {
-  HeapObject heap_object = UpdateTypedSlotHelper::GetTargetObject(
-      chunk->heap(), slot_type, slot_address);
+                                                Address slot_address,
+                                                MaybeObject new_target) {
+  Tagged<HeapObject> heap_object;
 
-#if DEBUG
-  UpdateTypedSlotHelper::UpdateTypedSlot(
-      chunk->heap(), slot_type, slot_address,
-      [heap_object](FullMaybeObjectSlot slot) {
-        DCHECK_EQ((*slot).GetHeapObjectAssumeStrong(), heap_object);
-        return KEEP_SLOT;
-      });
-#endif  // DEBUG
-
-  if (heap_object.InSharedWritableHeap()) {
+  if (new_target.GetHeapObject(&heap_object) &&
+      heap_object.InWritableSharedSpace()) {
     const uintptr_t offset = slot_address - chunk->address();
     DCHECK_LT(offset, static_cast<uintptr_t>(TypedSlotSet::kMaxOffset));
 
@@ -896,7 +895,7 @@ void RootScavengeVisitor::VisitRootPointers(Root root, const char* description,
 }
 
 void RootScavengeVisitor::ScavengePointer(FullObjectSlot p) {
-  Object object = *p;
+  Tagged<Object> object = *p;
   DCHECK(!HasWeakHeapObjectTag(object));
   DCHECK(!MapWord::IsPacked(object.ptr()));
   if (Heap::InYoungGeneration(object)) {
