@@ -46,7 +46,7 @@ class UserResourceControllerHost;
 class VisitedLinksManagerQt;
 class WebContentsAdapterClient;
 
-class Q_WEBENGINECORE_PRIVATE_EXPORT ProfileAdapter : public QObject
+class Q_WEBENGINECORE_EXPORT ProfileAdapter : public QObject
 {
 public:
     explicit ProfileAdapter(const QString &storagePrefix = QString());
@@ -133,8 +133,8 @@ public:
         NotificationPermission = 2,
         AudioCapturePermission = 3,
         VideoCapturePermission = 4,
-        ClipboardRead = 5,
-        ClipboardWrite = 6,
+        ClipboardReadWrite = 5,
+        LocalFontsPermission = 6,
     };
 
     enum PermissionState {
@@ -143,6 +143,18 @@ public:
         DeniedPermission = 2
     };
 
+    enum ClientHint : uchar {
+        UAArchitecture,
+        UAPlatform,
+        UAModel,
+        UAMobile,
+        UAFullVersion,
+        UAPlatformVersion,
+        UABitness,
+        UAFullVersionList,
+        UAWOW64,
+    };
+
     HttpCacheType httpCacheType() const;
     void setHttpCacheType(ProfileAdapter::HttpCacheType);
 
@@ -173,6 +185,12 @@ public:
     QString httpAcceptLanguage() const;
     void setHttpAcceptLanguage(const QString &httpAcceptLanguage);
 
+    QVariant clientHint(ClientHint clientHint) const;
+    void setClientHint(ClientHint clientHint, const QVariant &value);
+    bool clientHintsEnabled();
+    void setClientHintsEnabled(bool enabled);
+    void resetClientHints();
+
     void clearHttpCache();
 
 #if QT_CONFIG(ssl)
@@ -221,6 +239,7 @@ private:
     QHash<QByteArray, QPointer<QWebEngineUrlSchemeHandler>> m_customUrlSchemeHandlers;
     QHash<QByteArray, QWeakPointer<UserNotificationController>> m_ephemeralNotifications;
     QHash<QByteArray, QSharedPointer<UserNotificationController>> m_persistentNotifications;
+    bool m_clientHintsEnabled;
 
     QList<ProfileAdapterClient*> m_clients;
     QList<WebContentsAdapterClient *> m_webContentsAdapterClients;
