@@ -77,6 +77,10 @@ bool QQuickWebEngineSettings::javascriptCanOpenWindows() const
     To enable also the pasting of clipboard content from JavaScript,
     use javascriptCanPaste.
 
+    Since unrestricted clipboard access is a potential security concern, it is
+    recommended that applications leave this disabled and instead respond to
+    \l{WebEngineView::ClipboardReadWrite}{ClipboardReadWrite} feature permission requests.
+
     Disabled by default.
 */
 bool QQuickWebEngineSettings::javascriptCanAccessClipboard() const
@@ -383,6 +387,10 @@ bool QQuickWebEngineSettings::webRTCPublicInterfacesOnly() const
     Enables JavaScript \c{execCommand("paste")}.
     This also requires enabling javascriptCanAccessClipboard.
 
+    Since unrestricted clipboard access is a potential security concern, it is
+    recommended that applications leave this disabled and instead respond to
+    \l{WebEngineView::ClipboardReadWrite}{ClipboardReadWrite} feature permission requests.
+
     Disabled by default.
 */
 bool QQuickWebEngineSettings::javascriptCanPaste() const
@@ -461,6 +469,19 @@ bool QQuickWebEngineSettings::forceDarkMode() const
     return d_ptr->testAttribute(QWebEngineSettings::ForceDarkMode);
 }
 
+/*!
+    \qmlproperty bool WebEngineSettings::scrollAnimatorEnabled
+    \since QtWebEngine 6.8
+
+    Enables animated scrolling.
+
+    Disabled by default.
+ */
+bool QQuickWebEngineSettings::scrollAnimatorEnabled() const
+{
+    return d_ptr->testAttribute(QWebEngineSettings::ScrollAnimatorEnabled);
+}
+
 /*!
     \qmlproperty string WebEngineSettings::defaultTextEncoding
     \since QtWebEngine 1.2
@@ -475,6 +496,33 @@ QString QQuickWebEngineSettings::defaultTextEncoding() const
     return d_ptr->defaultTextEncoding();
 }
 
+ASSERT_ENUMS_MATCH(QQuickWebEngineSettings::AllowImageAnimation,
+                   QWebEngineSettings::AllowImageAnimation)
+ASSERT_ENUMS_MATCH(QQuickWebEngineSettings::AnimateImageOnce, QWebEngineSettings::AnimateImageOnce)
+ASSERT_ENUMS_MATCH(QQuickWebEngineSettings::DisallowImageAnimation,
+                   QWebEngineSettings::DisallowImageAnimation)
+/*!
+    \qmlproperty enumeration WebEngineSettings::imageAnimationPolicy
+    \since QtWebEngine 6.8
+
+    Specifies how an image animation should be handled when the image frames
+    are rendered for animation.
+
+    \value WebEngineSettings.AllowImageAnimation
+           Allows all image animations when the image frames are rendered.
+    \value WebEngineSettings.AnimateImageOnce
+           Animate the image once when the image frames are rendered.
+    \value WebEngineSettings.DisallowImageAnimation
+           Disallows all image animations when the image frames are rendered.
+
+    Default value is \c {WebEngineSettings.AllowImageAnimation}.
+*/
+QQuickWebEngineSettings::ImageAnimationPolicy QQuickWebEngineSettings::imageAnimationPolicy() const
+{
+    return static_cast<QQuickWebEngineSettings::ImageAnimationPolicy>(
+            d_ptr->imageAnimationPolicy());
+}
+
 ASSERT_ENUMS_MATCH(QQuickWebEngineSettings::DisallowUnknownUrlSchemes, QWebEngineSettings::DisallowUnknownUrlSchemes)
 ASSERT_ENUMS_MATCH(QQuickWebEngineSettings::AllowUnknownUrlSchemesFromUserInteraction, QWebEngineSettings::AllowUnknownUrlSchemesFromUserInteraction)
 ASSERT_ENUMS_MATCH(QQuickWebEngineSettings::AllowAllUnknownUrlSchemes, QWebEngineSettings::AllowAllUnknownUrlSchemes)
@@ -760,6 +808,14 @@ void QQuickWebEngineSettings::setForceDarkMode(bool on)
         Q_EMIT forceDarkModeChanged();
 }
 
+void QQuickWebEngineSettings::setScrollAnimatorEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(QWebEngineSettings::ScrollAnimatorEnabled);
+    d_ptr->setAttribute(QWebEngineSettings::ScrollAnimatorEnabled, on);
+    if (wasOn != on)
+        Q_EMIT scrollAnimatorEnabledChanged();
+}
+
 void QQuickWebEngineSettings::setUnknownUrlSchemePolicy(QQuickWebEngineSettings::UnknownUrlSchemePolicy policy)
 {
     QWebEngineSettings::UnknownUrlSchemePolicy oldPolicy = d_ptr->unknownUrlSchemePolicy();
@@ -782,6 +838,17 @@ void QQuickWebEngineSettings::setParentSettings(QQuickWebEngineSettings *parentS
     d_ptr->setParentSettings(parentSettings->d_ptr.data());
 }
 
+void QQuickWebEngineSettings::setImageAnimationPolicy(
+        QQuickWebEngineSettings::ImageAnimationPolicy policy)
+{
+    QWebEngineSettings::ImageAnimationPolicy oldPolicy = d_ptr->imageAnimationPolicy();
+    QWebEngineSettings::ImageAnimationPolicy newPolicy =
+            static_cast<QWebEngineSettings::ImageAnimationPolicy>(policy);
+    d_ptr->setImageAnimationPolicy(newPolicy);
+    if (oldPolicy != newPolicy)
+        Q_EMIT imageAnimationPolicyChanged();
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qquickwebenginesettings_p.cpp"
