@@ -411,6 +411,8 @@ static const QStringList expectedAPI = QStringList()
     << "QQuickWebEngineSettings.errorPageEnabledChanged() --> void"
     << "QQuickWebEngineSettings.forceDarkMode --> bool"
     << "QQuickWebEngineSettings.forceDarkModeChanged() --> void"
+    << "QQuickWebEngineSettings.scrollAnimatorEnabled --> bool"
+    << "QQuickWebEngineSettings.scrollAnimatorEnabledChanged() --> void"
     << "QQuickWebEngineSettings.focusOnNavigationEnabled --> bool"
     << "QQuickWebEngineSettings.focusOnNavigationEnabledChanged() --> void"
     << "QQuickWebEngineSettings.fullScreenSupportEnabled --> bool"
@@ -529,6 +531,7 @@ static const QStringList expectedAPI = QStringList()
     << "QQuickWebEngineView.CertificateErrorDomain --> ErrorDomain"
     << "QQuickWebEngineView.ChangeTextDirectionLTR --> WebAction"
     << "QQuickWebEngineView.ChangeTextDirectionRTL --> WebAction"
+    << "QQuickWebEngineView.ClipboardReadWrite --> Feature"
     << "QQuickWebEngineView.Comm10E --> PrintedPageSizeId"
     << "QQuickWebEngineView.ConnectionErrorDomain --> ErrorDomain"
     << "QQuickWebEngineView.Copy --> WebAction"
@@ -641,6 +644,7 @@ static const QStringList expectedAPI = QStringList()
     << "QQuickWebEngineView.LoadStartedStatus --> LoadStatus"
     << "QQuickWebEngineView.LoadStoppedStatus --> LoadStatus"
     << "QQuickWebEngineView.LoadSucceededStatus --> LoadStatus"
+    << "QQuickWebEngineView.LocalFontsAccess --> Feature"
     << "QQuickWebEngineView.MediaAudioCapture --> Feature"
     << "QQuickWebEngineView.MediaAudioVideoCapture --> Feature"
     << "QQuickWebEngineView.MediaVideoCapture --> Feature"
@@ -870,6 +874,11 @@ static const QStringList expectedAPI = QStringList()
     << "QWebEngineWebAuthPinRequest.error --> QWebEngineWebAuthUxRequest::PinEntryError"
     << "QWebEngineWebAuthPinRequest.minPinLength --> int"
     << "QWebEngineWebAuthPinRequest.remainingAttempts --> int"
+    << "QQuickWebEngineSettings.AllowImageAnimation --> ImageAnimationPolicy"
+    << "QQuickWebEngineSettings.AnimateImageOnce --> ImageAnimationPolicy"
+    << "QQuickWebEngineSettings.DisallowImageAnimation --> ImageAnimationPolicy"
+    << "QQuickWebEngineSettings.imageAnimationPolicy --> QQuickWebEngineSettings::ImageAnimationPolicy"
+    << "QQuickWebEngineSettings.imageAnimationPolicyChanged() --> void"
     ;
 
 static bool isCheckedEnum(QMetaType t)
